# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2005-2006 TINY SPRL. (http://tiny.be) All Rights Reserved.
#
#    Created by Nicolas Bessi on 13.10.08.
#    Copyright (c) 2008 CamptoCamp. All rights reserved.
#    Copyright (c) 2009 Zikzakmedia S.L. (http://zikzakmedia.com) All Rights Reserved.
#                       Jordi Esteve <jesteve@zikzakmedia.com>
#    Copyright (C) 2012 Pexego Sistemas Informáticos. All Rights Reserved
#                       $Marta Vázquez Rodríguez$
#                       $Omar Castiñeira Saavedra$
#                  2013 Mariano Ruiz <mrsarm@gmail.com>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import time
from openerp.report import report_sxw
import rml_parse
from openerp.tools.translate import _

class print_invoice_list(rml_parse.rml_parse):
    """ Report that print invoices grouped by currency and type """
    _name = 'report.account.print_invoice_list'


    def __init__(self, cr, uid, name, context):
        super(print_invoice_list, self).__init__(cr, uid, name, context)
        #contain tuples of in invoices : (curreny, [browse records], currency_untaxed, currency_tax, currency_total)
        self.in_invoices = []
        #contain tuples of in refunds : (curreny, [browse records], currency_untaxed, currency_tax, currency_total)
        self.in_refunds = []
        #contain tuples of out invoices : (curreny, [browse records], currency_untaxed, currency_tax, currency_total)
        self.out_invoices = []
        #contain tuples of out refunds : (curreny, [browse records], currency_untaxed, currency_tax, currency_total)
        self.out_refunds = []
        self.localcontext.update({
            'time': time,
            'in_invoices': self.in_invoices,
            'in_refunds': self.in_refunds,
            'out_invoices': self.out_invoices,
            'out_refunds': self.out_refunds,
            'detailed_taxes': False,
            'get_periods':self.get_periods,
            'get_journals':self.get_journals,
            'get_tax_accounts':self.get_tax_accounts,
        })

    def get_periods(self, form):
        result=''
        if form.has_key('periods') and len(form['periods']):
            period_ids = ",".join([str(x) for x in form['periods'] if x])
            self.cr.execute("SELECT name FROM account_period WHERE id in (%s)" % (period_ids))
            res = self.cr.fetchall()
            len_res = len(res) 
            for r in res:
                if (r == res[len_res-1]):
                    result+=r[0]+". "
                else:
                    result+=r[0]+", "
        if form.get('state', '') in ('bydate', 'bycreatedate', 'all') and form.has_key('date_from') and form.has_key('date_to'):
            if form['state'] == 'bycreatedate':
                result += _('Create Date') + ' '
            result += self.formatLang(form['date_from'], date=True) + ' - ' + self.formatLang(form['date_to'], date=True) + ' '
        return str(result and result[:-1]) or ''

    def get_journals(self, form):
        result=''
        if form.has_key('journal_ids') and len(form['journal_ids']):
            journal_ids = ",".join([str(x) for x in form['journal_ids'] if x])
            self.cr.execute("SELECT name FROM account_journal WHERE id in (%s)" % (journal_ids))
            res = self.cr.fetchall()
            len_res = len(res) 
            for r in res:
                if (r == res[len_res-1]):
                    result+=r[0]+". "
                else:
                    result+=r[0]+", "
        return result

    def get_tax_accounts(self, form):
        result=''
        if form.has_key('account_tax_ids') and len(form['account_tax_ids']):
            account_tax_ids = ",".join([str(x) for x in form['account_tax_ids'] if x])
            self.cr.execute("SELECT name FROM account_account WHERE id in (%s)" % (account_tax_ids))
            res = self.cr.fetchall()
            len_res = len(res) 
            for r in res:
                if (r == res[len_res-1]):
                    result+=r[0]+". "
                else:
                    result+=r[0]+", "
        return result

    def set_context(self, objects, data, ids, report_type = None):
        """We do the grouping and processing of invoices"""

        invoice_obj = self.pool.get('account.invoice')
        range_state = data['form']['state']
        detailed_taxes = data['form']['detailed_taxes']
        if detailed_taxes:
            account_tax_ids = data['form']['account_tax_ids']
        else:
            account_tax_ids = []
        if data.get('model') and data['model'] == 'ir.ui.menu':
            invoice_types = []
            if data['form']['out_invoice']:
                invoice_types.append('out_invoice')
            if data['form']['out_refund']:
                invoice_types.append('out_refund')
            if data['form']['in_invoice']:
                invoice_types.append('in_invoice')
            if data['form']['in_refund']:
                invoice_types.append('in_refund')
            invoice_states = []
            if data['form']['draft']:
                invoice_states.append('draft')
            if data['form']['proforma']:
                invoice_states.append('proforma')
                invoice_states.append('proforma2')
            if data['form']['open']:
                invoice_states.append('open')
            if data['form']['paid']:
                invoice_states.append('paid')
            if data['form']['cancel']:
                invoice_states.append('cancel')
            where = [('company_id','=',data['form']['company_id'][0]),('type','in',invoice_types),('state','in',invoice_states)]
            if range_state in ['byperiod','all']:
                period_ids = data['form']['periods']
                where.append(('period_id','in',period_ids))
            if range_state in ['bydate','all','none']:
                where.append(('date_invoice','>=',data['form']['date_from']))
                where.append(('date_invoice','<=',data['form']['date_to']))
            if range_state in ['bycreatedate']:
                where.append(('create_date','>=',data['form']['date_from']))
                where.append(('create_date','<=',data['form']['date_to']))
            journal_ids = data['form']['journal_ids']
            if len(journal_ids)>0:
                where.append(('journal_id','in',journal_ids))
            #print where
            ids = invoice_obj.search(self.cr, self.uid, where)
            objects = invoice_obj.browse(self.cr, self.uid, ids)

            self.localcontext.update({
                'detailed_taxes': detailed_taxes,
                'account_tax_ids': account_tax_ids,
            })

        if not ids :
            return super(print_invoice_list, self).set_context(objects, data, ids, report_type)

        if not isinstance(ids, list) :
            ids = [ids]

        # we create temp list that will be used for store invoices by type
        ininv = []
        outinv = []
        inref = []
        outref = []

        # we get the invoices and sort them by types
        invoices = invoice_obj.browse(self.cr, self.uid, ids)
        for inv in invoices:
            if inv.type == ('in_invoice'):
                ininv.append(inv)
            if inv.type == ('in_refund'):
                inref.append(inv)
            if inv.type == ('out_invoice'):
                outinv.append(inv)
            if inv.type == ('out_refund'):
                outref.append(inv)

        # we process the invoice and attribute them to the property
        order_by = data.get('form') and data.get('form').get('order_by') or None
        self.filter_invoices(ininv, self.in_invoices, detailed_taxes, account_tax_ids, order_by)
        self.filter_invoices(outinv, self.out_invoices, detailed_taxes, account_tax_ids, order_by)
        self.filter_invoices(inref, self.in_refunds, detailed_taxes, account_tax_ids, order_by)
        self.filter_invoices(outref, self.out_refunds, detailed_taxes, account_tax_ids, order_by)
        super(print_invoice_list, self).set_context(objects, data, ids, report_type)

    def __calc_tax(self, inv_group_taxes, tax_lines, account_tax_ids):
        inc_untaxed = inc_tax = inc_total = 0
        for tax_line in tax_lines:
            if len(account_tax_ids)==0 or tax_line.account_id.id in account_tax_ids:
                tax = None
                for t in inv_group_taxes:
                    if t['name'].lower() == tax_line.name.lower():
                        tax = t
                        break
                if not tax:
                    tax = {'name': tax_line.name, 'base': 0.0, 'amount': 0.0, 'total': 0.0}
                    inv_group_taxes.append(tax)
                tax['base']+=tax_line.base
                inc_untaxed+=tax_line.base
                tax['amount']+=tax_line.amount
                inc_tax+=tax_line.amount
                tax['total']+=tax_line.base+tax_line.amount
                inc_total+=tax_line.base+tax_line.amount
        return inc_untaxed, inc_tax, inc_total

    def filter_invoices(self, in_list, dest, detailed_taxes, account_tax_ids, order_by=None):
        if not in_list:
            return
        tmp = {}

        # Sort by invoice "number", "date + number" or "partner + reference"
        if order_by=='date':
            in_list.sort(key=lambda inv: "%s_%s" % (inv.date_invoice, inv.number))
        if order_by=='create_date':
            in_list.sort(key=lambda inv: "%s_%s" % (inv.create_date, inv.number))
        elif order_by=='partner':
            in_list.sort(key=lambda inv: "%s_%s" % (inv.partner_id and inv.partner_id.name, inv.reference))
        elif order_by=='number':
            in_list.sort(key=lambda inv: inv.number)

        # Group by currency
        for inv in in_list:
            currency = inv.currency_id.name
            if tmp.has_key(currency):
                tmp[currency].append(inv)
            else :
                tmp[currency] = [inv]

        # Compute the total for each currency group
        for curr in tmp:
            untaxed = full_untaxed = tax = total = full_total = 0
            inv_group_taxes = []
            for tmpinv in tmp[curr]:
                if detailed_taxes:
                    inc_untaxed, inc_tax, inc_total = self.__calc_tax(inv_group_taxes, tmpinv.tax_line, account_tax_ids)
                    untaxed+=inc_untaxed
                    tax+=inc_tax
                    total+=inc_total
                    full_untaxed+=tmpinv.amount_untaxed
                    full_total+=tmpinv.amount_total
                else:
                    untaxed += tmpinv.amount_untaxed
                    tax += tmpinv.amount_tax
                    total += tmpinv.amount_total
            # Append the tuple to the property
            dest.append((curr, tmp[curr], untaxed, tax, total, full_untaxed, full_total, inv_group_taxes))
        del tmp


report_sxw.report_sxw('report.account.invoice.list.report.wzd', 'account.invoice', 'addons/account_financial_report/report/invoice_list.rml', parser=print_invoice_list, header=False)
