# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Account Move Edit Reconciled
#    Copyright (C) 2013  Enterprise Objects Consulting
#
#    Author: Mariano Ruiz <mrsarm@gmail.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _

class account_move_line(osv.osv):
    _inherit = "account.move.line"

    def _update_check(self, cr, uid, ids, context=None):
        done = {}
        for line in self.browse(cr, uid, ids, context=context):
            err_msg = _('Move name (id): %s (%s)') % (line.move_id.name, str(line.move_id.id))
            if line.move_id.state <> 'draft' and (not line.journal_id.entry_posted):
                raise osv.except_osv(_('Error !'), _('You can not do this modification on a confirmed entry! ' \
                                                     'You can just change some non legal fields or you must unconfirm the journal entry first! \n%s') % err_msg)
            if line.reconcile_id:
                if not line.journal_id.edit_reconciled or not self._is_manager_user(cr, uid):
                    raise osv.except_osv(_('Error !'), _('You can not do this modification on a reconciled entry! ' \
                                                         'You can just change some non legal fields or you must unreconcile first!\n%s') % err_msg)
                self.pool.get('account.move.reconcile').log_check_reconcile_edited(cr, uid, line.reconcile_id.id, context)
            t = (line.journal_id.id, line.period_id.id)
            if t not in done:
                self._update_journal_check(cr, uid, line.journal_id.id, line.period_id.id, context)
                done[t] = True
        return True

    def _is_manager_user(self, cr, uid, context=None):
        user = self.pool.get('res.users').browse(cr, uid, uid)
        manager_group_id = self.pool.get('res.groups').search(cr,uid,[('name','=','Manager'),('category_id.name', '=', 'Accounting & Finance')])[0]
        for user_group in user.groups_id:
            if user_group.id == manager_group_id:
                return True
        return False

account_move_line()


class account_journal(osv.osv):
    _inherit = 'account.journal'

    _columns = {
       'edit_reconciled':  fields.boolean('Allow Update of Reconciled Entries',
                                        help="Allows to make modifications on a reconciled entry"),
    }

account_journal()


class account_move_reconcile(osv.osv):
    _inherit = "account.move.reconcile"

    def log_check_reconcile_edited(self, cr, uid, ids, context=None):
        if isinstance(ids, (list,tuple)):
            ids = ids[0]
        rec = self.browse(cr, uid, ids, context=context)
        message = _("Warning ! A Journal Item from the Account Reconciliation '%s' was edited, " \
                  "you should check to see if the counterpart needs to be reviewed") % (rec.name,)
        self.log(cr, uid, ids, message, context)

account_move_reconcile()
