# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Account Move Extra Validations
#    Copyright (C) 2014  Enterprise Objects Consulting
#
#    Author: Mariano Ruiz <mrsarm@gmail.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import osv
from openerp.tools.translate import _

class account_account(osv.osv):
    _inherit = "account.account"

    def get_account_parent(self, cr, uid, account, context=None):
        node = account
        while node.parent_id:
            node = node.parent_id
        return node if node != account else None

class account_move(osv.osv):
    _inherit = "account.move"

    def validate(self, cr, uid, ids, context=None):
        account_obj = self.pool.get("account.account")

        # This is for performance optimization in
        # in batch operations with account move, like
        # close all move from a period
        accounts_parents = {}

        for move in self.browse(cr, uid, ids, context):
            parent_id = None
            for line in move.line_id:
                if line.account_id.id not in accounts_parents:
                    line_parent_id = account_obj.get_account_parent(cr, uid, line.account_id, context).id
                    accounts_parents[line.account_id.id] = line_parent_id
                else:
                    line_parent_id = accounts_parents[line.account_id.id]
                if not parent_id:
                    parent_id = line_parent_id
                elif parent_id != line_parent_id:
                    raise osv.except_osv(_('Error'), _("Couldn't create move between different Charts of Accounts"))
        return super(account_move, self).validate(cr, uid, ids, context)

account_move()


class account_move_line(osv.osv):
    _inherit = "account.move.line"

    def reconcile(self, cr, uid, ids, type='auto', writeoff_acc_id=False,
                    writeoff_period_id=False, writeoff_journal_id=False, context=None):

        account_obj = self.pool.get("account.account")

        # This is for performance optimization in
        # in batch operations with account move, like
        # close all move from a period
        accounts_parents = {}

        parent_id = None
        for line in self.browse(cr, uid, ids, context=context):
            if line.account_id.id not in accounts_parents:
                line_parent_id = account_obj.get_account_parent(cr, uid, line.account_id, context).id
                accounts_parents[line.account_id.id] = line_parent_id
            else:
                line_parent_id = accounts_parents[line.account_id.id]
            if not parent_id:
                parent_id = line_parent_id
            elif parent_id != line_parent_id:
                raise osv.except_osv(_('Error'), _("To reconcile the entries Account Chart should be the same for all entries"))

        return super(account_move_line, self).reconcile(cr, uid, ids, type,
                      writeoff_acc_id, writeoff_period_id, writeoff_journal_id, context)

account_move_line()
