# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, MRP BoM Extra Fields
#    Copyright (C) 2013 Mariano Ruiz (Enterprise Objects Consulting)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields
import decimal_precision as dp

class mrp_bom(osv.osv):
    _inherit = 'mrp.bom'

    def _get_extra_fields(self, cr, uid, ids, field_name, arg, context={}):
        vals = {}
        for bom in self.browse(cr,uid,ids,context=context):
            try:
                if bom.bom_template:
                    vals[bom.id] = {
                            'standard_price': bom.product_tmpl_id.standard_price,
                            'list_price': bom.product_tmpl_id.list_price or False,
                            'qty_available': False
                    }
                    continue
            except AttributeError:
                # If 'bom_variant_multi' are not installed, the fields
                # 'bom_template' not exist, so an exception is raised when try to access them
                pass
            vals[bom.id] = {
                    'standard_price': bom.product_id.standard_price,
                    'list_price': bom.product_id.lst_price or False,
                    'qty_available': bom.product_id.qty_available
            }
        return vals

    _columns = {
        'standard_price': fields.function(_get_extra_fields, string='Cost Price', type="float", method=True, multi=True,
                                          digits_compute=dp.get_precision('Purchase Price'),
                                          help="Product's cost for accounting stock valuation. It is the base price for the supplier price."),
        'list_price': fields.function(_get_extra_fields,string='Sale Price', type="float", method=True, multi=True,
                                          digits_compute=dp.get_precision('Sale Price'),
                                          help="Base price for computing the customer price. Sometimes called the catalog price."),
        'qty_available': fields.function(_get_extra_fields, string='Quantity On Hand', type='float', method=True, multi=True),
    }

mrp_bom()
