# -*- coding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2012  Enterprise Objects Consulting
#
#    Author: Mariano Ruiz <mrsarm@gmail.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv
from osv import fields
from tools.translate import _
from tools import config
import decimal_precision as dp


class account_voucher(osv.osv):
    _inherit = "account.voucher"
    _columns= {
        'payment_rate': fields.float('Exchange Rate', digits_compute=dp.get_precision('Account Voucher Payment'), required=True, readonly=True, states={'draft': [('readonly', False)]},
            help='The specific rate that will be used, in this voucher, between the selected currency (in \'Payment Rate Currency\' field)  and the voucher currency.'),
    }

    def onchange_date(self, cr, uid, ids, date, currency_id, payment_rate_currency_id, amount, company_id, context=None):
        """
        Update the amount to pay with the payment_rate of the date
        """
        res = super(account_voucher, self).onchange_date(cr, uid, ids, date, currency_id, payment_rate_currency_id, amount, company_id, context)
        payment_rate = res['value'].get('payment_rate')
        if payment_rate != None and payment_rate != 1.0:
            res['value'].update({'amount' : context['default_amount'] / payment_rate})
        return res

account_voucher()
