# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Invoice Multi Shop
#    Copyright (C) 2013 Enterprise Objects Consulting
#                       http://www.eoconsulting.com.ar
#    Authors: Mariano Ruiz <mrsarm@gmail.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv


class account_invoice(osv.osv):
    _inherit = "account.invoice"

    def _get_shop_id(self, cr, uid, context={}):
        """ To get  Shop  for this order
        @return: Shop id """
        if context.get('type', False) in ['out_invoice', 'out_refund'] \
                or (context.get('active_model', False) == 'stock.picking' \
                    and context.get('default_type', False) == 'out'):
            user = self.pool.get('res.users').browse(cr, uid, uid, context)
            if user.shop_id:
                shop_id = user.shop_id.id
            elif user.shop_ids:
                shop_id = user.shop_ids[0].id
            else:
                res = self.pool.get('sale.shop').search(cr, uid, [])
                shop_id = res and res[0] or False
            return shop_id
        return False

    _columns = {
        'shop_id': fields.many2one('sale.shop', 'Shop', readonly=True, states={'draft': [('readonly', False)]}),
    }

    _defaults = {
        'shop_id': _get_shop_id,
    }

    def _check_shop_id(self, cr, uid, ids, context=None):
        inv = self.browse(cr, uid, ids[0], context=context)
        if not inv.shop_id and inv.type in ['out_invoice', 'out_refund']:
            return False
        return True

    def create(self, cr, uid, vals, context={}):
        if not vals.get('shop_id', False):
            if vals.get('type', '') == 'out_refund':
                if 'reference' in vals:
                    sale_order_obj = self.pool.get('sale.order')
                    reference = sale_order_obj.search(cr,uid,[('name','=',vals['reference'])])
                    if len(reference)>0:
                        shop_id = sale_order_obj.read(cr,uid,reference[0],['shop_id'])['shop_id']
                        if shop_id:
                            vals['shop_id'] = shop_id[0]
                if not vals.get('shop_id', False):
                    if not context.get('type', False):
                        ctx = context.copy()
                        ctx['type'] = vals['type']
                    vals['shop_id'] = self._get_shop_id(cr, uid, ctx)
            elif vals.get('type', '') == 'out_invoice':
                if context.get('shop_id', False):
                    if isinstance(context['shop_id'], (int,long)):
                        vals['shop_id'] = context['shop_id']
                    else:
                        vals['shop_id'] = context['shop_id'].id
                if not vals.get('shop_id', False):
                    if not context.get('type', False):
                        ctx = context.copy()
                        ctx['type'] = vals['type']
                    vals['shop_id'] = self._get_shop_id(cr, uid, ctx)

        return super(account_invoice,self).create(cr,uid,vals,context)

    _constraints = [
        (_check_shop_id, 'Error! The Shop cannot be null.', ['shop_id']),
    ]

account_invoice()


class account_invoice_line(osv.osv):
    _inherit = "account.invoice.line"

    def product_id_change(self, cr, uid, ids, product, uom, qty=0, name='',
                          type='out_invoice', partner_id=False, fposition_id=False,
                          price_unit=False, address_invoice_id=False,
                          currency_id=False, context={}, company_id=None):

        res = super(account_invoice_line, self).product_id_change(
                        cr, uid, ids, product, uom, qty, name,
                        type, partner_id, fposition_id,
                        price_unit, address_invoice_id,
                        currency_id, context, company_id)

        if product:
            user = self.pool.get('res.users').browse(cr, uid, uid)
            if user.shop_id.exclude_lines_notes and 'value' in res:
                if 'note' in res['value']:
                    del res['value']['note']
        return res

account_invoice_line()
