# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Stock Move Extension
#    Copyright (C) 2013 Enterprise Objects Consulting
#                       http://www.eoconsulting.com.ar
#    Authors: Mariano Ruiz <mrsarm@gmail.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _
import netsvc
import time

class stock_return_picking(osv.osv_memory):
    _inherit = 'stock.return.picking'

    origin_types = [('refund', 'Refund'),
                 ('warranty_replacement', 'Warranty Replacement'),
                 ('warranty_repair', 'Warranty Repair'),
                 ('repair', 'Repair'),
                 ('others', 'Others'),]

    _columns = {
        'origin': fields.selection(origin_types,'Origin',required=True),
     }

    def _get_origin_description(self, cr, uid, origin, context={}):
        for type_code, type_description in self.origin_types:
            if type_code == origin:
                return _(type_description)
        raise osv.except_osv(_('Error'), _('Origin "%s" not defined !.') % origin)

    def _get_origin_to_invoice(self, cr, uid, origin, context={}):
        property_obj = self.pool.get('ir.property')
        prop_ids = property_obj.search(cr, uid, [('name', '=', 'stock_return_' + origin)],context=context)
        if len(prop_ids)>0:
            return property_obj.read(cr,uid,prop_ids[0],['value_text'],context=context)['value_text']
        if origin in ('refund', 'repair'):
            return '2binvoiced'
        return 'none'

    def _get_origin_to_location_dest_id(self, cr, uid, origin, new_type, move, context={}):
        if new_type == 'in':
            property_obj = self.pool.get('ir.property')
            prop_ids = property_obj.search(cr, uid, [('name', '=', 'stock_return_location_dest_id_' + origin)],context=context)
            if len(prop_ids)>0:
                dest_id = int(property_obj.read(cr,uid,prop_ids[0],['value_reference'],context=context)['value_reference'].split(',')[1])
                if dest_id != move.location_dest_id.id:
                    return dest_id
        return move.location_id.id

    def onchange_origin(self,cr,uid,ids,origin,context):
        return {'value': { 'invoice_state': self._get_origin_to_invoice(cr,uid,origin,context) }}

    def create_returns(self, cr, uid, ids, context=None):
        """ 
         Creates return picking.
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param ids: List of ids selected
         @param context: A standard dictionary
         @return: A dictionary which of fields with values.
        """
        if context is None:
            context = {} 
        record_id = context and context.get('active_id', False) or False
        move_obj = self.pool.get('stock.move')
        pick_obj = self.pool.get('stock.picking')
        uom_obj = self.pool.get('product.uom')
        data_obj = self.pool.get('stock.return.picking.memory')
        wf_service = netsvc.LocalService("workflow")
        pick = pick_obj.browse(cr, uid, record_id, context=context)
        data = self.read(cr, uid, ids[0], context=context)
        date_cur = time.strftime('%Y-%m-%d %H:%M:%S')
        set_invoice_state_to_none = True
        returned_lines = 0
        
#        Create new picking for returned products
        if pick.type=='out':
            new_type = 'in'
        elif pick.type=='in':
            new_type = 'out'
        else:
            new_type = 'internal'
        seq_obj_name = 'stock.picking.' + new_type
        new_pick_name = self.pool.get('ir.sequence').get(cr, uid, seq_obj_name)
        ctx = context.copy()
        ctx['return'] = True
        if data['origin'] == 'others':
            origin = pick.name
        else:
            origin = '%s (%s)' % (self._get_origin_description(cr, uid, data['origin']), pick.name)
        new_picking = pick_obj.copy(cr, uid, pick.id, {
                'name': new_pick_name,
                'origin': origin,
                'move_lines':[], 'state':'draft', 'type':new_type,
                'date':date_cur, 'invoice_state':data['invoice_state'],}, context=ctx)
        
        val_id = data['product_return_moves']
        for v in val_id:
            data_get = data_obj.browse(cr, uid, v, context=context)
            mov_id = data_get.move_id.id
            new_qty = data_get.quantity
            move = move_obj.browse(cr, uid, mov_id, context=context)
            new_location = move.location_dest_id.id
            new_location_dest = self._get_origin_to_location_dest_id(cr, uid, data['origin'], new_type, move, context)
            returned_qty = move.product_qty
            for rec in move.move_history_ids2:
                returned_qty -= rec.product_qty

            if returned_qty != new_qty:
                set_invoice_state_to_none = False
            if new_qty:
                returned_lines += 1
                if move.date_expected < date_cur:
                    date_expected = date_cur
                else:
                    date_expected = move.date_expected
                new_move=move_obj.copy(cr, uid, move.id, {
                    'product_qty': new_qty,
                    'product_uos_qty': uom_obj._compute_qty(cr, uid, move.product_uom.id,
                        new_qty, move.product_uos.id),
                    'picking_id':new_picking, 'state':'draft',
                    'location_id':new_location, 'location_dest_id':new_location_dest,
                    'date':date_cur, 'date_expected': date_expected})
                move_obj.write(cr, uid, [move.id], {'move_history_ids2':[(4,new_move)]})
        if not returned_lines:
            raise osv.except_osv(_('Warning !'), _("Please specify at least one non-zero quantity!"))

        try:
            if pick.location_id or pick.location_dest_id:
                pick_obj.write(cr, uid, [new_picking],
                    {'location_id': new_location,
                     'location_dest_id': new_location_dest}, context=context)
        except AttributeError:
            pass
            # If 'stock_picking_simple_locations' module are not installed,
            # the fields 'location_id' and 'location_dest_id' in picking not exist,
            # so an exception is raised when try to access them

        if set_invoice_state_to_none:
            pick_obj.write(cr, uid, [pick.id], {'invoice_state':'none'})
        wf_service.trg_validate(uid, 'stock.picking', new_picking, 'button_confirm', cr)
        pick_obj.force_assign(cr, uid, [new_picking], context)
        # Update view id in context, lp:702939
        view_list = {
                'out': 'view_picking_out_tree',
                'in': 'view_picking_in_tree',
                'internal': 'vpicktree',
            }
        data_obj = self.pool.get('ir.model.data')
        res = data_obj.get_object_reference(cr, uid, 'stock', view_list.get(new_type, 'vpicktree'))
        context.update({'view_id': res and res[1] or False})
        return {
            'domain': "[('id', 'in', ["+str(new_picking)+"])]",
            'name': 'Picking List',
            'view_type':'form',
            'view_mode':'tree,form',
            'res_model':'stock.picking',
            'type':'ir.actions.act_window',
            'context':context,
        }

stock_return_picking()
